diff mbox

[2/2] hw/arm/virt-acpi-build: Add DBG2 table

Message ID 1441635826-4866-3-git-send-email-leif.lindholm@linaro.org
State New
Headers show

Commit Message

Leif Lindholm Sept. 7, 2015, 2:23 p.m. UTC
Add a DBG2 table, describing the pl011 UART.

Signed-off-by: Leif Lindholm <leif.lindholm@linaro.org>
---
 hw/arm/virt-acpi-build.c | 37 ++++++++++++++++++++++++++++++++++++-
 1 file changed, 36 insertions(+), 1 deletion(-)

Comments

Andrew Jones Sept. 7, 2015, 3:59 p.m. UTC | #1
On Mon, Sep 07, 2015 at 03:23:46PM +0100, Leif Lindholm wrote:
> Add a DBG2 table, describing the pl011 UART.
> 
> Signed-off-by: Leif Lindholm <leif.lindholm@linaro.org>
> ---
>  hw/arm/virt-acpi-build.c | 37 ++++++++++++++++++++++++++++++++++++-
>  1 file changed, 36 insertions(+), 1 deletion(-)
> 
> diff --git a/hw/arm/virt-acpi-build.c b/hw/arm/virt-acpi-build.c
> index 9088248..6d53dbe 100644
> --- a/hw/arm/virt-acpi-build.c
> +++ b/hw/arm/virt-acpi-build.c
> @@ -352,6 +352,38 @@ build_rsdp(GArray *rsdp_table, GArray *linker, unsigned rsdt)
>  }
>  
>  static void
> +build_dbg2(GArray *table_data, GArray *linker, VirtGuestInfo *guest_info)
> +{
> +    AcpiDebugPort2 *dbg2;
> +    const MemMapEntry *uart_memmap = &guest_info->memmap[VIRT_UART];
> +
> +    dbg2 = acpi_data_push(table_data, sizeof(*dbg2));
> +
> +    dbg2->debug_devices_offset = 44;

I'd prefer using offsetof(), even though it's unlikely to ever change.

> +    dbg2->number_debug_devices = 1;
> +    dbg2->debug_devices[0].revision = 0;
> +    dbg2->debug_devices[0].length = 40;

If we keep the debug device structure definition, then this can be
sizeof()

> +    dbg2->debug_devices[0].number_generic_address_registers = 1;
> +    dbg2->debug_devices[0].namespace_string_length = 2;
> +    dbg2->debug_devices[0].namespace_string_offset = 38;

Could use offsetof() if we have the debug device struct

> +    dbg2->debug_devices[0].oem_data_length = 0;
> +    dbg2->debug_devices[0].oem_data_offset = 0;
> +    dbg2->debug_devices[0].port_type = 0x8000;    /* Serial */
> +    dbg2->debug_devices[0].port_subtype = 0x3;    /* ARM PL011 UART */
> +    dbg2->debug_devices[0].base_address_register_offset = 22;

offsetof()

> +
> +    dbg2->debug_devices[0].base_address[0].space_id = AML_SYSTEM_MEMORY;
> +    dbg2->debug_devices[0].base_address[0].bit_width = 8;
> +    dbg2->debug_devices[0].base_address[0].bit_offset = 0;
> +    dbg2->debug_devices[0].base_address[0].access_width = 1;

We're missing a bunch of cpu_to_le*'s for the >1 byte, non-zero assignments
above.

> +    dbg2->debug_devices[0].base_address[0].address = cpu_to_le64(uart_memmap->base);
> +
> +    strcpy((char *)dbg2->debug_devices[0].namespace_string, ".");
> +
> +    build_header(linker, table_data, (void *)dbg2, "DBG2", sizeof(*dbg2), 0);
> +}
> +
> +static void
>  build_spcr(GArray *table_data, GArray *linker, VirtGuestInfo *guest_info)
>  {
>      AcpiSerialPortConsoleRedirection *spcr;
> @@ -577,7 +609,7 @@ void virt_acpi_build(VirtGuestInfo *guest_info, AcpiBuildTables *tables)
>      dsdt = tables_blob->len;
>      build_dsdt(tables_blob, tables->linker, guest_info);
>  
> -    /* FADT MADT GTDT MCFG SPCR pointed to by RSDT */
> +    /* FADT MADT GTDT MCFG DBG2 SPCR pointed to by RSDT */
>      acpi_add_table(table_offsets, tables_blob);
>      build_fadt(tables_blob, tables->linker, dsdt);
>  
> @@ -591,6 +623,9 @@ void virt_acpi_build(VirtGuestInfo *guest_info, AcpiBuildTables *tables)
>      build_mcfg(tables_blob, tables->linker, guest_info);
>  
>      acpi_add_table(table_offsets, tables_blob);
> +    build_dbg2(tables_blob, tables->linker, guest_info);
> +
> +    acpi_add_table(table_offsets, tables_blob);
>      build_spcr(tables_blob, tables->linker, guest_info);
>  
>      /* RSDT is pointed to by RSDP */
> -- 
> 2.1.4
> 
>

Thanks,
drew
diff mbox

Patch

diff --git a/hw/arm/virt-acpi-build.c b/hw/arm/virt-acpi-build.c
index 9088248..6d53dbe 100644
--- a/hw/arm/virt-acpi-build.c
+++ b/hw/arm/virt-acpi-build.c
@@ -352,6 +352,38 @@  build_rsdp(GArray *rsdp_table, GArray *linker, unsigned rsdt)
 }
 
 static void
+build_dbg2(GArray *table_data, GArray *linker, VirtGuestInfo *guest_info)
+{
+    AcpiDebugPort2 *dbg2;
+    const MemMapEntry *uart_memmap = &guest_info->memmap[VIRT_UART];
+
+    dbg2 = acpi_data_push(table_data, sizeof(*dbg2));
+
+    dbg2->debug_devices_offset = 44;
+    dbg2->number_debug_devices = 1;
+    dbg2->debug_devices[0].revision = 0;
+    dbg2->debug_devices[0].length = 40;
+    dbg2->debug_devices[0].number_generic_address_registers = 1;
+    dbg2->debug_devices[0].namespace_string_length = 2;
+    dbg2->debug_devices[0].namespace_string_offset = 38;
+    dbg2->debug_devices[0].oem_data_length = 0;
+    dbg2->debug_devices[0].oem_data_offset = 0;
+    dbg2->debug_devices[0].port_type = 0x8000;    /* Serial */
+    dbg2->debug_devices[0].port_subtype = 0x3;    /* ARM PL011 UART */
+    dbg2->debug_devices[0].base_address_register_offset = 22;
+
+    dbg2->debug_devices[0].base_address[0].space_id = AML_SYSTEM_MEMORY;
+    dbg2->debug_devices[0].base_address[0].bit_width = 8;
+    dbg2->debug_devices[0].base_address[0].bit_offset = 0;
+    dbg2->debug_devices[0].base_address[0].access_width = 1;
+    dbg2->debug_devices[0].base_address[0].address = cpu_to_le64(uart_memmap->base);
+
+    strcpy((char *)dbg2->debug_devices[0].namespace_string, ".");
+
+    build_header(linker, table_data, (void *)dbg2, "DBG2", sizeof(*dbg2), 0);
+}
+
+static void
 build_spcr(GArray *table_data, GArray *linker, VirtGuestInfo *guest_info)
 {
     AcpiSerialPortConsoleRedirection *spcr;
@@ -577,7 +609,7 @@  void virt_acpi_build(VirtGuestInfo *guest_info, AcpiBuildTables *tables)
     dsdt = tables_blob->len;
     build_dsdt(tables_blob, tables->linker, guest_info);
 
     /* FADT MADT GTDT MCFG SPCR pointed to by RSDT */
     acpi_add_table(table_offsets, tables_blob);
     build_fadt(tables_blob, tables->linker, dsdt);
 
@@ -591,6 +623,9 @@  void virt_acpi_build(VirtGuestInfo *guest_info, AcpiBuildTables *tables)
     build_mcfg(tables_blob, tables->linker, guest_info);
 
     acpi_add_table(table_offsets, tables_blob);
+    build_dbg2(tables_blob, tables->linker, guest_info);
+
+    acpi_add_table(table_offsets, tables_blob);
     build_spcr(tables_blob, tables->linker, guest_info);
 
     /* RSDT is pointed to by RSDP */
@@ -577,7 +609,7 @@  void virt_acpi_build(VirtGuestInfo *guest_info, AcpiBuildTables *tables)
     build_dsdt(tables_blob, tables->linker, guest_info);
 
-    /* FADT MADT GTDT MCFG SPCR pointed to by RSDT */
+    /* FADT MADT GTDT MCFG DBG2 SPCR pointed to by RSDT */
     acpi_add_table(table_offsets, tables_blob);
     build_fadt(tables_blob, tables->linker, dsdt);